import base64
import datetime
from hashlib import sha1
from typing import Any, List

from suds import WebFault
from suds.client import Client

from dfn_pki_client import objects as obj
from dfn_pki_client.utils import get_wsdl


class PublicServicePKI:

    def __init__(self, config_path: str = 'config.ini', proxy: dict = {}):
        self.wsdl = get_wsdl('public', config_path)
        self.proxy = proxy
        self.client = Client(self.wsdl, proxy=self.proxy)

    def new_request(self, ra_id: int, pkcs10: Any, names: List, role: str, pin: str,
                    add_name: str, add_email: str, add_org_unit: str, publish: bool) -> int:
        """4.1.1 newRequest
        Args:
            ra_id (int): Nummer der RA, 0 für die Master-RA (RaID)
            pkcs10 (str): Der Zertifikatantrag im PEM-Format (PKCS10)
            names (List): Subject Alternative Names in der Form ("typ:wert", …) (AltNames)
            role (str): Die Rolle des beantragten Zertifikats (Role)
            pin (str): Das Sperrkennwort für das Zertifikat als SHA-1 Hash (Pin)
            add_name (str): Vollständiger Name des Antragstellers (AddName)
            add_email (str): E-Mail Adresse des Antragstellers (AddEMail)
            add_org_unit (str): Abteilung des Antragstellers (AddOrgUnit)
            publish (bool): Veröffentlichung des Zertifikats (Publish)

        Returns:
            serial (int): Die Seriennummer des hochgeladenen Antrags
        """

        alt_names = self.client.factory.create('ArrayOfString')
        alt_names._arrayType = "ns0:ArrayOfString"
        alt_names.item = ['DNS:{}'.format(url) for url in names]
        alt_names = ''

        pin = sha1(pin.encode()).hexdigest()

        try:
            new_request = self.client.service.newRequest(
                RaID=ra_id,
                PKCS10=pkcs10,
                AltNames=alt_names,
                Role=role,
                Pin=pin,
                AddName=add_name,
                AddEMail=add_email,
                AddOrgUnit=add_org_unit,
                Publish=publish
            )
        except WebFault as web_fault:
            print(f"Error: {web_fault}")
            new_request = None

        return new_request

    def new_revocation_request(self, ra_id: int, serial: int, reason: str, pin: str) -> int:
        """4.1.2 newRevocationRequest

        Args:
            ra_id (int): Nummer der RA, 0 für die Master-RA (RaID)
            serial (int): Die Seriennummer des Zertifikats (Serial)
            reason (str): Der Grund für die Sperrung (Reason)
            pin (str): Das Sperrkennwort für das Zertifikat als SHA-1 Hash (Pin)

        Returns:
            serial (int): Die Seriennummer des neuen Sperrantrags
        """

        pin = sha1(pin.encode()).hexdigest()

        return self.client.service.newRevocationRequest(ra_id, serial, reason, pin)

    def get_request_printout(self, ra_id: int, serial: int, pin: str, file: str = '') -> bytes:
        """4.1.3 getRequestPrintout
        Args:
            ra_id (int): Nummer der RA, 0 für die Master-RA (RaID)
            serial (int): Die Seriennummer des Zertifikatantrags
            # format (str): Rückgabeformat (Format)
            pin (str): Das Sperrkennwort für das Zertifikat als SHA-1 Hash (Pin)

        Returns:
            binary (bytes): Der Ausdruck des Zertifikatantrags

        """

        pin = sha1(pin.encode()).hexdigest()

        pdf = self.client.service.getRequestPrintout(ra_id, serial, 'application/pdf', pin)

        if not file:
            now = datetime.datetime.now()
            timestamp = now.strftime('%d_%m_%Y_%H_%M_%S')
            file = f"report_{timestamp}.pdf"

        with open(file, 'wb') as pdf_file:
            pdf_file.write(base64.b64decode(pdf))

        return pdf

    def get_certificate_by_request_serial(self, ra_id: int, serial: int, pin: str) -> str:
        """4.1.4 getCertificateByRequestSerial
        Args:
            ra_id (int): Nummer der RA, 0 für die Master-RA (RaID)
            serial (int): Die Seriennummer des Zertifikats (Serial)
            pin (str): Das Sperrkennwort für das Zertifikat als SHA-1 Hash

        Returns:
            cert (str): Das ausgestellte Zertifikat im PEM-Format

        """

        pin = sha1(pin.encode()).hexdigest()

        return self.client.service.getCertificateByRequestSerial(ra_id, serial, pin)

    def get_valid_domains(self, ra_id: int, type: str = '') -> List:
        """4.1.5 getValidDomains (RaID Type)
        Args:
            ra_id (int): Nummer der RA, 0 für die Master-RA (RaID)
            type (str): Domain-Typ: 'server' oder 'email' (Type)
        Returns:
            Liste mit Domain-Einträgen
        """
        if type:
            return self.client.service.getValidDomains(ra_id, type)
        else:
            return self.client.service.getValidDomains(ra_id)

    def get_request_info(self, ra_id: int, serial: int, pin: str) -> obj.DFNCERTTypesRequestInfo:
        """4.1.6 getRequestInfo
        args:
            ra_id (int): Nummer der RA (RaID)
            serial (int): Seriennummer des Zertifikatantrags (Serial)
            pin (str): SHA1 der Sperr-PIN des Zertifikatantrags (Hex-String) (Pin)

        Returns:
            Rückgabe: Struktur mit Informationen über den Antrag
        """

        res = self.client.service.getRequestInfo(ra_id, serial, pin)

        param = res.Parameters
        parameters = obj.DFNCERTTypesRequestParameters(param.RaID, param.Subject, param.SubjectAltNames, param.Role,
                                                       param.NotBefore, param.NotAfter, param.AdditionalName,
                                                       param.AdditionalEMail, param.AdditionalUnit)

        return obj.DFNCERTTypesRequestInfo(res.Serial, res.SameDNSerials, res.Status, parameters, res.PublicKey,
                                           res.PublicKeyAlgorithm, res.PublicKeyDigest, res.PublicKeyLength,
                                           res.Publish, res.SignatureAlgorithm, res.DateSubmitted, res.DateApproved,
                                           res.DateDeleted)

    def get_ca_info(self, ra_id: int):
        """4.1.7 getCAInfo
        Args:
            ra_id (int): Nummer der RA

        Returns:
            Rückgabe: Struktur mit Informationen über die CA
        """

        res = self.client.service.getCAInfo(ra_id)
        ra_infos = res.RAInfos[0]
        ra_info = obj.DFNCERTypesRAInfo(ra_infos.ID, ra_infos.Name, ra_infos.DNPrefixes)
        return obj.DFNCERTypesCAInfo(res.RALoginID, ra_info, res.CAChain, res.Roles)



import configparser
import sys
import urllib

from ssl import PROTOCOL_TLS as ssl_protocol
from typing import List

from OpenSSL.crypto import load_pkcs12
from suds.client import Client
from suds.transport.http import HttpTransport
from urllib3.contrib.pyopenssl import PyOpenSSLContext

from dfn_pki_client import objects as obj
from dfn_pki_client.utils import get_wsdl
from dfn_pki_client.client_authentication import SudsTransport


class DomainService:

    def __init__(self, config_path: str = 'config.ini', proxy: dict = {}):
        self.wsdl = get_wsdl('domain', config_path)
        self.proxy = proxy
        self.client = Client(self.wsdl, transport=SudsTransport(config_path, self.proxy))

    def delete_domain(self, ra_id: int, name: str, type: str, change: str) -> str:
        """5.5.4 deleteDomain

        Args:
            ra_id (int): Die RA des Domain-Namen, der gelöscht werden soll
            name (str): Name des zu löschenden Domain-Eintrags
            type (str): Typ des Domain-Eintrags (server[-host] oder email[-host])
            change (str): Letzte Änderungsprüfsumme

        Returns:
            Rückgabe (str): Aktuelle Änderungsprüfsumme
        """

        return self.client.service.deleteDomain(ra_id, name, type, change)

    def delete_domain2(self, ra_id: int, name: str, type: str, change: str) -> obj.DFNCERTTypesDeleteDomain2Result:
        """5.5.5 deleteDomain2

        Args:
            ra_id (int): Die RA des Domain-Namen, der gelöscht werden soll
            name (str): Name des zu löschenden Domain-Eintrags
            type (str): Typ des Domain-Eintrags (server[-host] oder email[-host])
            change (str): Letzte Änderungsprüfsumme

        Returns:
            Rückgabe (DFNCERTTypesDeleteDomain2Result): Aktuelle Änderungsprüfsumme und ggf. Liste der gültigen Zertifikate zu diesem Domain-Namen.
        """

        res = self.client.service.deleteDomain2(ra_id, name, type, change)
        return obj.DFNCERTTypesDeleteDomain2Result(res.Result, res.CertInfos)

    def get_tlds(self) -> obj.DFNCERTTypesTLDs:
        """5.5.6 getTLDs

        Returns:
            Rückgabe (DFNCERTTypesTLDs): Liste der Top-level-domains
        """

        res = self.client.service.getTLDs()
        return obj.DFNCERTTypesTLDs(res.TLDs)

    def get_certificates_for_domain(self, ra_id: int, name: str, type: str, status: str) -> List[obj.DFNCERTTypesShortCertInfo]:
        """5.5.7 getCertificatesForDomain

        Args:
            ra_id (int): Die RA des Domain-Namen (RaID)
            name (str): Domain-Name (Name)
            type (str): Typ des Domain-Eintrags (server[-host] oder email[-host]) (Type)
            status (str): Status der Zertifikate ('VALID' oder 'REVOKED') (Status)

        Returns:
            Rückgabe (List[DFNCERTTypesShortCertInfo]): Liste der gültigen Zertifikate zu diesem Domain-Namen.
        """

        res = self.client.service.getCertificatesForDomain(ra_id, name, type, status)
        short_cert_info = []
        for r in res:
            short_cert_info.append(obj.DFNCERTTypesShortCertInfo(r.RaID, r.Serial, r.SubjectDN, r.NotAfter))
        return short_cert_info

    def get_validation_parameter(self, name: str) -> List[obj.DFNCERTTypesValidationParameter]:
        """5.5.8 getValidationParameter

        Args:
            name (str): Domain-Name

        Returns:
            Rückgabe (List[DFNCERTTypesValidationParameter]): Liste der möglichen Validierungs-Parameter für die angefragte Domain.

        """

        res = self.client.service.getValidationParameter(name)
        print(res)
        validation_parameter = []
        for r in res:
            name = r.Name if hasattr(r, 'Name') else ''
            email = r.EMail if hasattr(r, 'EMail') else ''
            adns = r.ADNs if hasattr(r, 'ADNs') else ''
            validation_parameter.append(obj.DFNCERTTypesValidationParameter(name, r.Method, email, adns))
        return validation_parameter

    def list_domains(self, ra_id: int) -> obj.DFNCERTTypesDomainListResult:
        """5.5.1 listDomains

         Args:
             ra_id (int): Die RA deren Domain-Namen gelistet werden sollen

        Returns:
            Rückgabe (DFNCERTTypesDomainListResult): Alle Domain-Einträge und die Zugriffsrechte der angeforderten RA
        """

        res = self.client.service.listDomains(ra_id)
        results = []
        for result in res.Result:
            results.append(obj.DFNCERTTypesDomain(result.Name, result.Type, result.Secret,
                                                  result.Approved, result.ApprovedDate))
        acl = obj.DFNCERTTypesDomainACL(res.ACL.RaID, res.ACL.Allowed)
        return obj.DFNCERTTypesDomainListResult(res.Change, results, acl)

    def list_extended_domains(self, ra_id: int) -> obj.DFNCERTTypesExtendedDomainListResult:
        """5.5.2 listExtendedDomains

        Args:
            ra_id (int): Die RA, deren Domain-Namen gelistet werden sollen

        Returns:
            Rückgabe (DFNCERTTypesExtendedDomainListResult): Alle Domain-Einträge und die Zugriffsrechte der angeforderten RA
        """

        res = self.client.service.listExtendedDomains(ra_id)
        print(res)
        results = []
        for result in res.Result:
            results.append(obj.DFNCERTTypesDomain(result.Name, result.Type, result.Secret,
                                                  result.Approved, result.ApprovedDate))
        acl = obj.DFNCERTTypesDomainACL(res.ACL.RaID, res.ACL.Allowed)
        return obj.DFNCERTTypesExtendedDomainListResult(results, acl)

    def request_domain(self, ra_id: int, name: str, type: str, public: bool, change: str) -> str:
        """5.5.3 requestDomain

        Args:
            ra_id (int): Die RA in der ein Domain-Name beantragt werden sollen
            name (str): Beantragter Domain-Name
            type (str): Typ des Domain-Eintrags (server[-host] oder email[-host])
            public (bool): Sichtbar auf den Antragsseiten
            change (str): Aktuelle Änderungsprüfsumme

        Returns:
            Rückgabe (str): Neue Änderungsprüfsumme

        values of type:
            server: Es wird ein Domain-Name für Server beantragt. Es sollen alle Hostnamen inklusive Subdomains vor diesem Namen erlaubt sein.
            server-host: Es wird genau ein FQDN für Serverzertifikate erlaubt.
            email: Es wird ein Domain-Name für EMail-Adressen beantragt. Es sind alle Adressen vor dieser Domain und beliebige Subdomains zugelassen. Beispiel: Eingetragen wird „dfn.de“. Gültige Adressen sind dann „pki@dfn.de“ sowie „pki@pca.dfn.de“.
            email-host: Es wird genau eine Domain für EMail-Adressen beantragt. Es sind beliebige EMail-Adressen aber keine beliebigen Subdomains erlaubt.
        """

        return self.client.service.requestDomain(ra_id, name, type, public, change)

    def set_validation_parameter(self, ra_id: int, name: str, type: str, method: str,
                                 email_local: str, email_domain: str, change: str) -> str:
        """5.5.9 setValidationParameter

        Args:
            ra_id (int): RA-ID des Domain-Eintrags
            name (str): Domain-Name
            type (str): Typ (server, server-host)
            method (str): Prüfmethode (2-Domain-Contact-Mail-SOA, 2-Domain-Contact-Mail-Whois, 4-Constructed-Mail)
            email_local (str): Lokaler Part der E-Mail-Adresse, an die eine Challenge-E-Mail versendet werden soll.
            email_domain (str): Domain Part der E-Mail-Adresse, an die eine Challenge-E-Mail versendet werden soll.
            change (str): Letzte Änderungsprüfsumme

        Returns:
            Rückgabe (str): Aktuelle Änderungsprüfsumme
        """

        return self.client.service.setValidationParameter(ra_id, name, type, method, email_local, email_domain, change)

    def send_challenge_email(self, ra_id: int, name: str, type: str, change: str) -> obj.DFNCERTTypesSendChallengeEMailResult:
        """5.5.10 sendChallengeEMail

        Args:
            ra_id (int): RA-ID des Domain-Eintrags
            name (str): Domain-Name
            type (str): Typ (server, server-host)
            change (str): Letzte Änderungsprüfsumme

        Returns:
            Rückgabe (DFNCERTTypesSendChallengeEMailResult): Struktur, die die aktualle Änderungsprüfsumme sowie das Datum, an dem die Challenge-E-Mail versendet wurde, enthält.
        """

        res = self.client.service.sendChallengeEMail(ra_id, name, type, change)
        return obj.DFNCERTTypesSendChallengeEMailResult(res.Change, res.LastChallengeEMailSent)



from typing import List, Union

from suds.client import Client

from dfn_pki_client import client_authentication
from dfn_pki_client import objects as obj
from dfn_pki_client.utils import get_wsdl


class RegistrationService:

    def __init__(self, config_path: str, proxy: dict = {}):
        self.wsdl = get_wsdl('registration', config_path)
        self.proxy = proxy
        self.client = Client(self.wsdl, transport=client_authentication.SudsTransport(proxy))

    def get_ca_status(self) -> obj.DFNCERTTypesCAStatus:
        """5.1.1 getCAStatus

        Returns:
            Rückgabe (DFNCERTTypesCAStatus): Struktur mit Informationen über neue Elemente (tns:DFNCERTTypesCAStatus)
        """

        res = self.client.service.getCAStatus()
        return obj.DFNCERTTypesCAStatus(res.RequestNewCount, res.RevocationNewCount)

    def get_ca_info(self) -> obj.DFNCERTypesCAInfo:
        """5.1.2 getCAInfo

        Returns:
            Rückgabe (DFNCERTypesCAInfo): Struktur mit Informationen über die CA (tns:DFNCERTTypesCAInfo)
        """

        res = self.client.service.getCAInfo()
        ra_infos = []
        for info in res.RAInfos:
            info_id = info[2][0]
            info_name = info[1][0]
            info_dn_prefixes = info[0][0]
            ra_infos.append(obj.DFNCERTypesRAInfo(info_id, info_name, info_dn_prefixes))

        return obj.DFNCERTypesCAInfo(res.RALoginID, ra_infos, res.CAChain, res.Roles)

    def search_items_2(self, type: str, status: str, role: Union[str, None], desired_ra_id: Union[int, None],
                       last_serial: Union[int, None], limit: int) -> List[obj.DFNCERTTypesExtendedObjectInfo]:
        """5.1.3 searchItems2

        Args:
            type (str): Die Art der Einträge, die gesucht werden sollen
            status (str): Der Status der gesuchten Einträge
            role (str): Die Rolle der gesuchten Einträge. Darf 'null' sein, dann wird die Suche nicht nach der Rolle eingeschränkt.
            desired_ra_id (int): Die RA-ID, für die Einträge zurückgeliefert werden sollen. Darf 'null' sein, dann werden alle Einträge, für die der angemeldete RA-Operator eine Berechtigung hat, zurückgegeben.
            last_serial (int): Seriennummer, ab der die Suche fortgesetzt werden soll. Darf 'null' sein, dann wird bei der größten Seriennummer gestartet.
            limit (int): Die Anzahl der Einträge die zurückgeliefert werden sollen

        Returns:
            Liste von Informationsobjekten (List[DFNCERTTypesExtendedObjectInfo]) (tns:DFNCERTTypesExtendedObjectInfo[])

        # Type: [request | crr] -> Status: [NEW | PENDING | APPROVED | DELETED | ARCHIVED]
        # Type: certificate -> Status: [VALID, REVOKED]

        Diese Funktion ersetzt searchItems, searchExtendedItems, searchItemsByRole sowie searchItemsForRaID.
        """

        res = self.client.service.searchItems2(type, status, role, desired_ra_id, last_serial, limit)
        information_objects = []
        for r in res:
            not_before = r.NotBefore if hasattr(r, 'NotBefore') else None
            information_objects.append(obj.DFNCERTTypesExtendedObjectInfo(
                r.Serial, r.Subject, r.EMail, r.Role[0], r.Date, r.UnconfirmedEMails,
                r.RaID, r.AdditionalName, r.AdditionalEMail, r.AdditionalUnit, not_before
            ))
        return information_objects

    def get_request_data(self, serial: int) -> obj.DFNCERTTypesRequestData:
        """5.1.8 getRequestData

        Args:
            serial (int): Seriennummer des Zertifikatantrags (serial)

        Returns:
            Rückgabe (DFNCERTTypesRequestData): Struktur mit Informationen über den Zertifikatantrag (tns1:DFNCERTTypesRequestData)
        """

        res = self.client.service.getRequestData(serial)
        return obj.DFNCERTTypesRequestData(res.Serial, res.RaID, res.PKCS10,
                                           res.AltNames, res.Role, res.AddName,
                                           res.AddEMail, res.AddOrgUnit, res.Publish)

    def approve_request(self, serial: int, content: bytes, signature: str) -> bool:
        """5.2.1 approveRequest

        Args:
            serial (int): Die Seriennummer des Zertifikatantrags
            content (byte): Der signierte Inhalt, Rückgabe von getRawRequest (xsd:base64Binary)
            signature (str): PKCS#7 mit Signatur über den Antrag im PEM-Format

        Returns:
            Rückgabe (bool): Bei Erfolg true
        """

        return self.client.service.approveRequest(serial, content, signature)

    def delete_request(self, serial: int) -> bool:
        """5.2.2 deleteRequest

        Args:
            serial (int): Die Seriennummer des Zertifikatantrags (Serial)

        Returns:
            Rückgabe (bool): Bei Erfolg true
        """

        return self.client.service.deleteRequest(serial)

    def renew_request(self, serial) -> int:
        """5.2.3 renewRequest

        Args:
            serial (int): Die Seriennummer des Zertifikatantrags

        Returns:
            Rückgabe (int): Seriennummer des erneuerten Antrags
        """

        return self.client.service.renewRequest(serial)

    def renew_request_set_publish_if_needed(self, serial: int) -> obj.DFNCERTTypesRenewRequestResult:
        """5.2.4 renewRequestSetPublishIfNeeded

        Args:
            serial (int): Die Seriennummer des Zertifikatantrags (Serial)

        Returns:
            Rückgabe (DFNCERTTypesRenewRequestResult): Datenstruktur mit Angaben zum erneuerten Antrag (Tns1:DFNCERTTypesRenewRequestResult)
        """

        res = self.client.service.renewRequestSetPublishIfNeeded(serial)
        return obj.DFNCERTTypesRenewRequestResult(res.Serial, res.Server, res.Publish, res.HasChanged)

    def get_raw_request(self, serial: int) -> bytes:
        """5.2.5 getRawRequest

        Args:
            serial (int): Die Seriennummer des Zertifikatantrags (Serial)

        Returns:
            Rückgabe: Der komplette zu signierende Zertifikatantrag (xsd:base64Binary)
        """

        return self.client.service.getRawRequest(serial)

    def get_request_info(self, serial: int) -> obj.DFNCERTTypesRequestInfo:
        """5.2.6 getRequestInfo
        Args:
            serial (int): Die Seriennummer des Zertifikatantrags (Serial)

        Returns:
            Rückgabe (DFNCERTTypesRequestInfo): Struktur mit Informationen über den Antrag (tns:DFNCERTTypesRequestInfo)
        """

        res = self.client.service.getRequestInfo(serial)
        return obj.DFNCERTTypesRequestInfo(
            res.Serial, res.SameDNSerials, res.Status, res.Parameters,
            res.PublicKey, res.PublicKeyAlgorithm, res.PublicKeyDigest,
            res.PublicKeyLength, res.Publish, res.SignatureAlgorithm,
            res.DateSubmitted, res.DateApproved, res.DateDeleted
        )

    def get_extended_request_info(self, serial: int) -> obj.DFNCERTTypesExtendedRequestInfo:
        """5.2.7 getExtendedRequestInfo

        Args:
            serial (int): Die Seriennummer des Zertifikatantrags (Serial)

        Returns:
            Rückgabe (DFNCERTTypesExtendedRequestInfo): Struktur mit Informationen über den Antrag (tns:DFNCERTTypesExtendedRequestInfo)
        """

        res = self.client.service.getExtendedRequestInfo(serial)
        return obj.DFNCERTTypesExtendedRequestInfo(
            res.Serial, res.SameDNSerials, res.Status, res.Parameters, res.PublicKey, res.PublicKeyAlgorithm,
            res.PublicKeyDigest, res.PublicKeyLength, res.Publish, res.SignatureAlgorithm, res.DateSubmitted,
            res.DateApproved, res.DateDeleted, res.SignerCertificateSerial, res.SignerCN
        )

    def get_request_printout(self, serial: int, format: str = 'application/pdf') -> bytes:
        """5.2.8 getRequestPrintout

        Args:
            serial (int): Die Seriennummer des Zertifikatantrags (Serial)
            format (str): Das gewünschte Format (MIME-Type) des Ausdrucks (Format)

        Returns:
            Rückgabe: Der Ausdruck des Zertifikatantrags (xsd:base64Binary)
        """

        return self.client.service.getRequestPrintout(serial, format)

    def set_request_parameters(self, serial: int, request_parameters) -> bool:
        """5.2.9 setRequestParameters

        Args:
            serial (int): Die Seriennummer des Zertifikatantrags (Serial)
            request_parameters: Eine Struktur mit gewünschten Werten

        Returns:
            Rückgabe (bool): Bei Erfolg true
        """

        return self.client.service.setRequestParameters(serial, request_parameters)

    def set_extended_request_parameters(self, serial: int, request_parameters) -> bool:
        """5.2.10 setExtendedRequestParameters

        Args:
            serial (int): Die Seriennummer des Zertifikatantrags (Serial)
            request_parameters: Eine Struktur mit gewünschten Werten

        Returns:
            Rückgabe (bool): Bei Erfolg true
        """

        return self.client.service.setRequestParameters(serial, request_parameters)

    def send_confirmation_email(self, serial, emails: List) -> bool:
        """5.2.11 sendConfirmationEMail

        Args:
            serial (int): Die Seriennummer des Zertifikatantrags (Serial)
            emails (List[str]): Liste mit E-Mail-Adressen, an die eine Bestätigungs-E-Mail versendet werden soll

        Returns:
            Rückgabe (bool): Bei Erfolg true
        """

        return self.client.service.sendConfirmationEMail(serial, emails)

    def get_certificate(self, serial: int) -> str:
        """5.3.1 getCertificate

        Args:
            serial (int): Die Seriennummer des Zertifikats

        Returns:
            Rückgabe (str): Das gewünschte Zertifikat im PEM-Format
        """
        return self.client.service.getCertificate(serial)

    def get_certificate_by_request_serial(self, serial: int) -> str:
        """5.3.2 getCertificateByRequestSerial

        Args:
            serial (int): Die Seriennummer des Zertifikatantrags

        Returns:
            Rückgabe (str): Das gewünschte Zertifikat im PEM-Format
        """

        return self.client.service.getCertificateByRequestSerial(serial)

    def get_certificate_info(self, serial: int) -> obj.DFNCERTTypesCertificateInfo:
        """5.3.3 getCertificateInfo

        Args:
            serial (int): Die Seriennummer des Zertifikats

        Returns:
            Rückgabe (DFNCERTTypesCertificateInfo): Meta-Informationen über das Zertifikat
        """

        res = self.client.service.getCertificateInfo(serial)
        return obj.DFNCERTTypesCertificateInfo(res.RequestSerial, res.Publish, res.Role, res.Status, res.PEM)

    def new_revocation_request(self, serial: int, reason: str) -> int:
        """5.4.1 newRevocationRequest

        Args:
            serial (int): Die Seriennummer des Zertifikats
            reason (str): Der Gurnd für die Speerung

        Returns:
            Rückgabe (int): Die Seriennummer des neuen Sperrantrags
        """

        return self.client.service.newRevocationRequest(serial, reason)

    def approve_revocation_request(self, serial: int, content: bytes, signature: str) -> bool:
        """5.4.2 approveRevocationRequest

        Args:
            serial (int): Die Seriennummer des Sperrantrags
            content (bytes): Rückgabewert von getRawRevocationRequest
            signature (str): PKCS#7 Signatur über den Sperrantrag

        Returns:
            Rückgabe (bool): Bei Erfolg true
        """

        return self.client.service.approveRevocationRequest(serial, content, signature)

    def get_raw_revocation_request(self, serial: int) -> bytes:
        """5.4.3 getRawRevocationRequest

        Args:
            serial (int): Die Seriennummer des Sperrantrags

        Returns:
            Rückgabe (bytes): Sperrantrag aus der Datenbank der CA
        """

        return self.client.service.getRawRevocationRequest(serial)

    def get_revocation_info(self, serial: int) -> obj.DFNCERTTypesRevocationInfo:
        """5.4.4 getRevocationInfo

        Args:
            serial (int): Die Seriennummer des Sperrantrags

        Returns:
            Rückgabe (DFNCERTTypesRevocationInfo):
        """

        res = self.client.service.getRevocationInfo(serial)
        return obj.DFNCERTTypesRevocationInfo(res.Status, res.Serial, res.CertificateSerial, res.RaID,
                                              res.DateSubmitted, res.DateApproved, res.DateDeleted,
                                              res.Parameters)



#!/usr/bin/env python

import click

from public_service import PublicServicePKI
from registration_service import RegistrationService
from utils import get_wsdl


@click.command()
@click.version_option()
def main():

    pass


if __name__ == '__main__':
    main()



from dataclasses import dataclass
from typing import List

from suds.sax.date import DateTime
from suds.sax.text import Text


@dataclass
class DFNCERTTypesCAStatus:
    """6.1.1 DFNCERTTypesCAStatus

    Attributes:
        request_new_count (int): Anzahl der neuen Zertifikatanträge (RequestNewCount)
        revocation_new_count (int): Anzahl der neuen Sperranträge (RevocationNewCount)
    """
    request_new_count: int
    revocation_new_count: int


@dataclass
class DFNCERTypesRAInfo:
    """6.1.3 DFNCERTypesRAInfo

    Attributes:
        id (int): RA-Nummer des Eintrags (ID)
        name (str): Installationsname der CA (Name)
        dn_prefixes (List[Text]): Liste mit allen erlaubten Namensräumen (DNPrefixes)
    """
    id: int
    name: str
    dn_prefixes: List[Text]


@dataclass
class DFNCERTypesCAInfo:
    """6.1.2 DFNCERTypesCAInfo

    Attributes:
        ra_login_id (int): RA_ID des Clients nach Authentifizierung (RALoginID)
        ra_infos (DFNCERTypesRAInfo): Liste mit Informationen über alle zu dieser CA gehörenden Registrierungsstellen (RAInfos)
        ca_chain (str): Das aktuelle CA-Zertifikat und Kette im PEM-Format (CAChain)
        roles (str): Liste mit allen von dieser CA unterstützten Rollen-Namen (Roles)
    """
    ra_login_id: int
    ra_infos: List[DFNCERTypesRAInfo]
    ca_chain: str
    roles: str


@dataclass
class DFNCERTTypesCertificateInfo:
    """6.2.1 DFNCERTTypesCertificateInfo

    Attributes:
        request_serial (int): Seriennummer des passenden Requests zu diesem Zertifikat (RequestSerial)
        publish (bool): Wurde das Zertifikat veröffentlicht? (Publish)
        role (str): Die Rolle des Zertifikats in der CA (Role)
        status (str): Der Status des Zertifikats (VALID oder REVOKED) (Status)
        pem (str): Das Zertifikat im PEM-Format (PEM)
    """
    request_serial: int
    publish: bool
    role: str
    status: str
    pem: str


@dataclass
class DFNCERTTypesShortCertInfo:
    """6.2.2 DFNCERTTypesShortCertInfo

    Attributes:
        ra_id (int): RA-Nummer des Zertifikats (RaID)
        serial (int): Seriennummer des Zertifikats (Serial)
        subject_dn (str): Der Subject-DN des Zertifikats (SubjectDN)
        not_after (DateTime): Das Ablaufdatum des Zertifikats („gültig bis“, Zeitzone: UTC) (NotAfter) (xsd:dateTime)

    """
    ra_id: int
    serial: int
    subject_dn: str
    not_after: DateTime


@dataclass
class DFNCERTTypesObjectInfo:
    """6.2.3 DFNCERTTypesObjectInfo

    Attributes:
        serial (int): Seriennummer des Eintrags (Serial)
        subject (str): Subject-DN des Eintrags (Subject)
        email (str): E-Mail Adresse entweder aus Subject-DN, wenn dort nicht vorhanden aus AdditionalEmail (EMail)
        role (str): Beantragte Rolle des Eintrags (Role)
        date (DateTime): Für Anträge Datum des Eingangs und für Zertifikate NotAfter (entspricht "gültig bis", Zeitzone: UTC) (xsd:dateTime)
    """
    serial: int
    subject: str
    email: str
    role: str
    date: DateTime


@dataclass
class DFNCERTTypesExtendedObjectInfo:
    """6.2.4 DFNCERTTypesExtendedObjectInfo

    Attributes:
        serial (int): Seriennummer des Eintrags (Serial)
        subject (str): Subject-DN des Eintrags (Subject)
        email (str): E-Mail Adresse entweder aus Subject-DN, wenn dort nicht vorhanden aus AdditionalEmail (EMail)
        role (str): Beantragte Rolle des Eintrags (Role)
        date (DateTime): Für Anträge Datum des Eingangs und für Zertifikate NotAfter (entspricht "gültig bis", Zeitzone: UTC) (xsd:dateTime) (Date)
        unconfirmed_emails (int): Anzahl der noch nicht bestätigten E-Mail-Adressen (UnconfirmedEMails)
        ra_id (int): RA Nummer des Eintrags (RaID)
        additional_name (str): Name des Antragsstellers (AdditionalName)
        additional_email (str): Kontakt-E-Mail-Adresse des Antragsstellers (AdditionalEMail)
        additional_org_unit (str): Abteilung des Antragsstellers (AdditionalOrgUnit)
        not_before (DateTime): Gültigkeitsbeginn des Zertifikats (Zeitzone: UTC) (NotBefore) (xsd:dateTime)
    """
    serial: int
    subject: str
    email: str
    role: str
    date: DateTime
    unconfirmed_emails: int
    ra_id: int
    additional_name: str
    additional_email: str
    additional_org_unit: str
    not_before: DateTime = None


@dataclass
class DFNCERTTypesRequestParameters:
    """6.3.1 DFNCERTTypesRequestParameters

    Attributes:
        ra_id (int): RA Nummer des Antrags (RaID)
        subject (str): Subject DN des Antrags (Subject)
        subject_alt_names (List[str]): Subject Alternative Names als Array von Strings. Format ist: ("typ:wert", …) (SubjectAltNames)
        role (str): Die Rolle des beantragten Zertifikats (Role)
        not_before (DateTime): Gültigkeitsbeginn des Zertifikats (Zeitzone: UTC) (xsd:dateTime) (NotBefore)
        not_after (DateTime): Gültigkeitsende des Zertifikats (Zeitzone: UTC) (xsd:dateTime) (NotAfter)
        additional_name (str): Name des Antragsstellers (AdditionalName)
        additional_email (str): E-Mail-Adresse des Antragstellers (AdditionalEMail)
        additional_unit (str): Abteilung des Antragsstellers (AdditionalUnit)
    """
    ra_id: int
    subject: str
    subject_alt_names: List[str]
    role: str
    not_before: DateTime
    not_after: DateTime
    additional_name: str
    additional_email: str
    additional_unit: str


@dataclass
class DFNCERTTypesExtendedRequestParameters:
    """6.3.2 DFNCERTTypesExtendedRequestParameters

    Attributes:
        ra_id (int): RA Nummer des Antrags (RaID)
        subject (str): Subject DN des Antrags (Subject)
        subject_alt_names (List[str]): Subject Alternative Names als Array von Strings. Format ist: ("typ:wert", …) (SubjectAltNames)
        role: (str): Die Rolle des beantragten Zertifikats (Roile)
        not_before (DateTime): Gültigkeitsbeginn des Zertifikats (Zeitzone: UTC) (xsd:dateTime)
        not_after (DateTime): Gültigkeitsende des Zertifikats (Zeitzone: UTC) (xsd:dateTime)
        additional_name (str): Name des Antragsstellers (AdditionalName)
        additional_email (str): Kontakt-E-Mail-Adresse des Antragstellers (AdditionalEMail)
        additional_unit (str): Abteilung des Antragsstellers (AdditionalUnit)
        validity_days (int): Gültigkeitsdauer des Zertifikats in Tagen (ValidityDays)
        email_addresses (List[str]): Liste mit allen E-Mail-Adressen des gegebenen Antrags, die in das Zertifikat aufgenommen werden sollen (für Anträge, die nach dem 1.7.2014 gestellt wurden)
    """

    ra_id: int
    subject: str
    subject_alt_names: List[str]
    role: str
    not_before: DateTime
    not_after: DateTime
    additional_name: str
    additional_email: str
    additional_unit: str
    validity_days: int
    mail_addresses: List[str]


@dataclass
class EMail:
    """6.3.3 Email

    Attributes:
        local (str): Lokaler-Part der E-Mail-Adresse (Local)
        domain (str): Domain-Part der E-Mail-Adresse (Domain)
        state (str): Status des Eintrags (EMailState)
        request_serial (int): Antragsnummer des zugehörigen Zertifikatantrags
        location (str): Ort, an dem die E-Mail-Adresse im Zertifikat steht (EMailLocation)
        last_send_date (DateTime): Zeitpunkt, an dem die letzte Bestätigungs-E-Mail versendet wurde (Zeitzone: UTC) (xsd:dateTime)
        state_change_date (DateTime): Zeitpunkt, an dem der State verändert wurde. (Zeitzone: UTC) (xsd:dateTime)

    Values of EMailState:
        PENDING: Bestätigung für die E-Mail-Adresse steht noch aus
        REJECTED: E-Mail-Adresse wurde zurückgewiesen
        CONFIRMED: E-Mail-Adresse wurde durch den Nutzer bestätigt
        WHITELISTED: E-Mail-Adresse wurde durch die Whitelist bestätigt

    Values of EMailLocation:
        DN: E-Mail-Adresse befindet sich im DN
        SAN: E-Mail-Adresse befindet sich im Subject-AltName
        DN_AND_SAN: E-Mail-Adresse sowohl im DN als auch im Subject-AltName
    """
    local: str
    domain: str
    state: str
    request_serial: int
    location: str
    last_send_date: DateTime
    state_change_date: DateTime


@dataclass
class DFNCERTTypesRequestInfo:
    """6.3.4 DFNCERTTypesRequestInfo

    Attributes:
        serial (int): Seriennummer des Antrags (Serial)
        same_dn_serials (List[int]): Liste mit Seriennummern von Zertifikaten, die den gleichen Subject DN tragen (SameDNSerials)
        status (str): Status des Eintrags (NEW, PENDING, RENEW, APPROVED, DELETED, ARCHIVED) (Status)
        parameters (str) : Struktur mit allen veränderbaren Parametern eines Zertifikatantrags (DFNCERTRequestParameters) (Parameters)
        public_key (str): Der öffentliche Schlüssel des Antrags in OpenSSL-Ausgabeformat (PublicKey)
        public_key_algorithm (str): Der verwendete Algorithmus des Schlüssels (PublicKeyAlgorithm)
        public_key_digest (str): Ein SHA-1 über den öffentlichen Schlüssel (PublicKeyDigest)
        public_key_length (int): Die Länge des öffentlichen Schlüssels in Bit (PublicKeyLength)
        publish (bool): Flagge für die Veröffentlichung des Antrags (Publish)
        signature_algorithm (str): Der verwendete Algorithmus bei der Signatur des Antrags (SignatureAlgorithm)
        date_submitted (str): Datum an dem der Antrag einging (Zeitzone: UTC) (DateSubmitted)
        date_approved (str): Das Datum der Genehmigung des Antrags (Zeitzone: UTC) (DateApproved)
        date_deleted (str): Das Datum der Löschung des Antrags (Zeitzone: UTC) (DateDeleted)

    values of status:
        NEW: Ein Zertifikatantrag (Request) ist im Zustand NEW, wenn er initial neu erzeugt worden ist.
        RENEW: Ein Request, der von einem achivierten Request abgeleitet worden ist ("Kopie").
        PENDING: Ein neuer oder erneuerter Request, der von einem CAO1 verändert worden ist.
        DELETED: Ein gelöschter Request. Von diesem wurde kein Zertifikat erzeugt.
        APPROVED: Ein freigegebener Request. Aus diesem Request basierend soll nun ein Zertifikat erzeugt werden.
        ARCHIVED: Ein Request, von dem ein Zertifikat erzeugt worden ist.

    """
    serial: int
    same_dn_serials: List[int]
    status: str
    parameters: DFNCERTTypesRequestParameters
    public_key: str
    public_key_algorithm: str
    public_key_digest: str
    public_key_length: int
    publish: bool
    signature_algorithm: str
    date_submitted: str
    date_approved: str
    date_deleted: str


@dataclass
class DFNCERTTypesExtendedRequestInfo:
    """6.3.5 DFNCERTTypesExtendedRequestInfo

    Attributes:
        serial (int): Seriennummer des Antrags (Serial)
        same_dn_serials (List[int]): Liste mit Seriennummern von Zertifikaten, die den gleichen Subject DN tragen (SameDNSerials)
        status (str): Status des Eintrags (s. DFNCERTTypesRequestInfo) (Status)
        parameters (tns:DFNCERTExtendedRequestParameters): Struktur mit allen veränderbaren Parametern (Parameters)
        public_key (str): Der öffentliche Schlüssel des Antrags in OpenSSL-Ausgabeformat (PublicKey)
        public_key_algorithm (str): Der verwendete Algorithmus des Schlüssels (PublicKeyAlgorithm)
        public_key_digest (str): Ein SHA-1 über den öffentlichen Schlüssel (PublicKeyDigest)
        public_key_length (int): Die Länge des öffentlichen Schlüssels in Bit (PublicKeyLength)
        publish (bool): Flagge für die Veröffentlichung des Antrags (Publish)
        signature_algorithm (str): Der verwendete Algorithmus bei der Signatur des Antrags (SignatureAlgorithm)
        date_submitted (str): Datum an dem der Antrag einging (DateSubmitted)
        date_approved (str): Das Datum der Genehmigung des Antrags (DateApproved)
        date_deleted (str): Das Datum der Löschung des Antrags (DateDeleted)
        signer_certificate_serial (int): Seriennummer des Unterzeichner-Zertifikats (SignerCertificateSerial)
        signer_cn (str): Subject-CN des Unterzeichner-Zertifikats (SignerCN)

    """
    serial: int
    same_dn_serials: List[int]
    status: str
    parameters: str
    public_key: str
    public_key_algorithm: str
    public_key_digest: str
    public_key_length: str
    publish: bool
    signature_algorithm: str
    date_submitted: str
    date_approved: str
    date_deleted: str
    signer_certificate_serial: int
    signer_cn: str


@dataclass
class DFNCERTTypesRequestData:
    """6.3.6 DFNCERTTypesRequestData

    Attributes:
        serial (int): Seriennummer des Antrags (Serial)
        ra_id (int): Nummer der RA, 0 für die Master-RA (RaID)
        pkcs10 (str): Der Zertifikatantrag im PEM-Format (PKCS10)
        alt_names (List[str]): Subject Alternative Names in der Form ("typ:wert", …) (tns1:ArrayOfString) (AltNames)
        role (str): Die Rolle des beantragten Zertifikats (Role)
        add_name (str): Vollständiger Name des Antragstellers (AddName)
        add_email (str): E-Mail Adresse des Antragstellers (AddEMail)
        add_org_unit (str): Abteilung des Antragstellers (AddOrgUnit)
        publish (bool): Veröffentlichung des Zertifikats (Publish)
    """
    serial: int
    ra_id: int
    pksc10: str
    alt_names: List[str]
    role: str
    add_name: str
    add_email: str
    add_org_unit: str
    publish: bool


@dataclass
class DFNCERTTypesRenewRequestResult:
    """6.3.7 DFNCERTTypesRenewRequestResult

    Attributes:
        serial (int): Seriennummer des Antrags (Serial)
        server (bool): Flag, das angibt, ob es sich um einen Antrag für ein Server-Zertifikat handelt. (Server)
        publish (bool): Veröffentlichung des Zertifikats (Publish)
        has_changed (bool): Flag, das angibt, ob der Wert von Publish beim erneuerten Antrag geändert wurde. (HasChanged)
    """
    serial: int
    server: bool
    publish: bool
    has_changed: bool


@dataclass
class DFNCERTTypesRevocationParameters:
    """6.4.1 DFNCERTTypesRevocationParameters

    Attributes:
        reason (str): Grund der Sperrung
    """
    reason: str


@dataclass
class DFNCERTTypesRevocationInfo:
    """6.4.2 DFNCERTTypesRevocationInfo

    Attributes:
        status (str): Status des Eintrags (Status)
        serial (int): Seriennummer des Eintrags (Serial)
        certificate_serial (int): Seriennummer des zu sperrenden Zertifikats (CertificateSerial)
        ra_id (int): RA-Nummer des Eintrags (RaID)
        date_submitted (str): Datum an dem der Antrag einging (Zeitzone: UTC) (DateSubmitted)
        date_approved (str): Das Datum der Genehmigung des Antrags (Zeitzone: UTC) (DateApproved)
        date_deleted (str): Das Datum der Löschung des Antrags (Zeitzone: UTC) (DateDeleted)
        parameters (DFNCERTTypesRevocationParameters): Parameter für einen Sperrantrag (Grund der Sperrung) (Parameters)
    """
    status: int
    serial: int
    certificate_serial: int
    ra_id: int
    date_submitted: str
    date_approved: str
    date_deleted: str
    parameters: DFNCERTTypesRevocationParameters


@dataclass
class DFNCERTTypesDomain:
    """6.5.1 DFNCERTTypesDomain

    Attributes:
        name (str): Domain-Name (Name)
        type (str): Typ (server, server-host, email oder email-host) (Type)
        secret (bool): Versteckt vor der Öffentlichkeit (Secret)
        approved (bool): Freigegeben (Approved)
        approved_date (DateTime): Freigabezeitpunkt (Zeitzone: UTC) (xsd:dateTime) (ApprovedDate)
    """
    name: str
    type: str
    secret: bool
    approved: bool
    approved_date: DateTime


@dataclass
class DFNCERTTypesExtendedDomain:
    """6.5.2 DFNCERTTypesExtendedDomain

    Attributes:
        name (str): Domain-Name
        type (str): Typ (server, server-host, email oder email-host)
        secret (bool): Versteckt vor der Öffentlichkeit
        approved (bool): Freigegeben
        approved_date (str): Freigabezeitpunkt (Zeitzone: UTC) (xsd:dateTime)
        method (str): Prüfmethode, nach der die Domain validiert wurde/werden soll
        br_version (str): Versionsnummer der Baseline-Requirements, auf die sich die Prüfmethode bezieht
        challange_mail_address (str): E-Mail-Adresse, an die die Challenge-E-Mail versendet wird/wurde
        last_challange_mail_sent (DateTime): Datum, an dem die letzte Challenge-E-Mail versendet wurde (Zeitzone: UTC) (xsd:dateTime)
        valid_until (DateTime): Gültigkeitsende (Zeitzone: UTC) (xsd:dateTime)
    """
    name: str
    type: str
    secret: bool
    approved: bool
    approved_date: DateTime
    method: str
    br_version: str
    challange_mail_address: str
    last_challange_mail_sent: DateTime
    valid_until: DateTime


@dataclass
class DFNCERTTypesDomainACL:
    """6.5.3 DFNCERTTypesDomainACL

    Attributes:
        ra_id (int): RA_ID für die diese Liste gilt (RaID)
        allowed (List[str]): Liste erlaubter Aktionen (Whitelist) (Allowed)
    """
    ra_id: int
    allowed: List[str]


@dataclass
class DFNCERTTypesDomainListResult:
    """6.5.4 DFNCERTTypesDomainListResult

    Attributes:
        change (str): Aktuelle Änderungsprüfsumme (Change)
        result (List[DFNCERTTypesDomain]): Liste mit gefundenen Domain-Einträgen (Result)
        acl (DFNCERTTypesDomainACL): Zugriffsrechte für angeforderte RA_ID (ACL)
    """
    change: str
    result: List[DFNCERTTypesDomain]
    acl: DFNCERTTypesDomainACL


@dataclass
class DFNCERTTypesExtendedDomainListResult:
    """6.5.5 DFNCERTTypesExtendedDomainListResult

    Attributes:
        result (List[DFNCERTTypesExtendedDomain]): Liste mit gefundenen Domain-Einträgen (Result)
        acl (DFNCERTTypesDomainACL): Zugriffsrechte für angeforderte RA_ID (ACL)
    """
    result: List[DFNCERTTypesExtendedDomain]
    acl: DFNCERTTypesDomainACL


@dataclass
class DFNCERTTypesDeleteDomain2Result:
    """6.5.6 DFNCERTTypesDeleteDomain2Result

    Attributes:
        change (str): Aktuelle Änderungsprüfsumme (Change)
        cert_infos (List[DFNCERTTypesShortCertInfo]): Liste mit gefundenen Zertifikat-Einträgen (CertInfos)
    """
    change: str
    cert_infos: List[DFNCERTTypesShortCertInfo]


@dataclass
class DFNCERTTypesTLDs:
    """6.5.7 DFNCERTTypesTLDs

    Attributes:
        tdls (List[str]): Liste mit den Top-level-domains (TLDs)
    """
    tdls: List[str]


@dataclass
class DFNCERTTypesValidDomain:
    """6.5.8 DFNCERTTypesValidDomain

    Attributes:
        name (str): Domain-Name
        type (str): Typ (server, server-host, email oder email-host)
    """
    name: str
    type: str


@dataclass
class DFNCERTTypesValidationParameter:
    """6.5.9 DFNCERTTypesValidationParameter

    Attributes:
        name (str): Domain-Name (Name)
        method (str): Prüfmethode (2-Domain-Contact-Mail-SOA oder 4-Constructed-Mail) (Method)
        email (str): E-Mail-Adresse, die zur Prüfmethode passt. (EMail)
        adns (List[str]): Liste der zur Prüfmethode passenden Authorization Domain Names (ADNs)

    Bzgl. Prüfmethoden siehe setValidationParameter.
    """
    name: str
    method: str
    email: str
    adns: List[str]


@dataclass
class DFNCERTTypesSendChallengeEMailResult:
    """6.5.10 DFNCERTTypesSendChallengeEMailResult

    Attributes:
        change (str): Aktuelle Änderungsprüfsumme (Change)
        last_challende_email_sent (DateTime): Datum, an dem die Challenge-E-Mail gesendet wurde. (Zeitzone: UTC) (xsd:dateTime) (LastChallengeEMailSent)
    """
    change: str
    last_challende_email_sent: DateTime



import configparser
import pathlib
import sys
import urllib
from ssl import PROTOCOL_TLS as ssl_protocol
from typing import Tuple

from OpenSSL.crypto import load_pkcs12
from suds.transport.http import HttpTransport
from urllib3.contrib.pyopenssl import PyOpenSSLContext


class SudsTransport(HttpTransport):
    def __init__(self, proxy=None, section: str = 'default'):
        self.section = section
        self.cert_name, self.password = self.get_cert_password(self.section)
        self.proxy = proxy
        self.context = self.create_pyopenssl_sslcontext()
        super(SudsTransport, self).__init__(proxy=self.proxy)

    @staticmethod
    def get_cert_password(section: str, config_path: str = 'config.ini') -> Tuple[str, str]:

        path = pathlib.Path(config_path)
        if not path.exists():
            print(f"File {path} doesn't exist.")
            sys.exit(-1)

        config = configparser.ConfigParser()
        config_exists = config.read(config_path)
        if config_exists:
            if section in config.sections():
                if config[section]['cert']:
                    cert_name = config[section]['cert']
                else:
                    print('Certfile is missing.')
                    sys.exit(-1)
                if config[section]['password']:
                    password = config[section]['password']
                else:
                    print('Password is missing.')
                    sys.exit(-1)
                return cert_name, password
            else:
                print(f"Section '{section}' in config.ini is missing.")
                sys.exit(-1)
        else:
            print(f"File {config_path} not found or is empty.")
            sys.exit(-1)

    def u2handlers(self):
        handlers = super(SudsTransport, self).u2handlers()
        handlers.append(urllib.request.HTTPSHandler(context=self.context))
        handlers.append(urllib.request.ProxyHandler(proxies=self.proxy))
        return handlers

    def create_pyopenssl_sslcontext(self):
        with open(self.cert_name, 'rb') as pkcs12_file:
            pkcs12_data = pkcs12_file.read()

        if isinstance(self.password, bytes):
            pkcs12_password_bytes = self.password
        else:
            pkcs12_password_bytes = self.password.encode('utf8')

        p12 = load_pkcs12(pkcs12_data, pkcs12_password_bytes)
        cert = p12.get_certificate()

        ssl_context = PyOpenSSLContext(ssl_protocol)
        ssl_context._ctx.use_certificate(cert)

        ca_certs = p12.get_ca_certificates()
        if ca_certs:
            for ca_cert in ca_certs:
                ssl_context._ctx.add_extra_chain_cert(ca_cert)
        ssl_context._ctx.use_privatekey(p12.get_privatekey())

        return ssl_context



import configparser
import pathlib
import sys
import typing

from OpenSSL.crypto import load_pkcs12
from ssl import PROTOCOL_TLS as ssl_protocol
from urllib3.contrib.pyopenssl import PyOpenSSLContext


def get_wsdl(wsdl: str, path: str = 'config.ini', section: str = 'default') -> typing.Optional[str]:
    config = configparser.ConfigParser()
    path = pathlib.Path(path)
    if path.exists():
        config.read(path)

        if section in config:
            if config[section][f"{wsdl}_wsdl"]:
                return config[section][f"{wsdl}_wsdl"]
            else:
                print(f"WSDL entry {wsdl} is missing.")
                sys.exit(-1)
        else:
            print(f"Section {section} in config.ini is missing.")
            sys.exit(-1)
    else:
        print(f"File {path} doesn't exist.")
        sys.exit(-1)


def gen_ssl_context(path: str, section: str = 'default'):
    config = configparser.ConfigParser()

    path = pathlib.Path(path)
    if not path.exists():
        print(f"File {path} doesn't exist.")
        sys.exit(-1)

    config.read(path)

    if section in config:
        if config[section]['cert']:
            cert_name = config[section]['cert']
        else:
            print('Certfile is missing.')
            sys.exit(-1)
        if config[section]['password']:
            password = config[section]['password']
        else:
            print('Password is missing.')
            sys.exit(-1)
    else:
        print(f"Section {section} in config.ini is missing.")
        sys.exit(-1)

    with open(cert_name, 'rb') as pkcs12_file:
        pkcs12_data = pkcs12_file.read()

    if isinstance(password, bytes):
        pkcs12_password_bytes = password
    else:
        pkcs12_password_bytes = password.encode('utf8')

    p12 = load_pkcs12(pkcs12_data, pkcs12_password_bytes)
    cert = p12.get_certificate()

    ssl_context = PyOpenSSLContext(ssl_protocol)
    ssl_context._ctx.use_certificate(cert)

    ca_certs = p12.get_ca_certificates()
    if ca_certs:
        for ca_cert in ca_certs:
            ssl_context._ctx.add_extra_chain_cert(ca_cert)
    ssl_context._ctx.use_privatekey(p12.get_privatekey())

    return ssl_context



from cryptography import x509
from cryptography.x509.oid import NameOID
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import hashes, serialization
from cryptography.hazmat.primitives.asymmetric import rsa


class OpenSSL:

    def __init__(self):
        pass

    @staticmethod
    def create_csr(file_name: str = 'DFN-PKI-Client.key', password: str = '') -> str:

        # create public and private key
        # https://www.daemonology.net/blog/2009-06-11-cryptographic-right-answers.html
        private_key = rsa.generate_private_key(public_exponent=65537, key_size=2048,
                                               backend=default_backend())

        if password:
            pem = private_key.private_bytes(
                encoding=serialization.Encoding.PEM,
                format=serialization.PrivateFormat.TraditionalOpenSSL,
                encryption_algorithm=serialization.BestAvailableEncryption(b'password')
            )
        else:
            pem = private_key.private_bytes(
                encoding=serialization.Encoding.PEM,
                format=serialization.PrivateFormat.PKCS8,
                encryption_algorithm=serialization.NoEncryption()
            )

        with open(file_name, 'wb') as pk_file:
            pk_file.write(pem)

        # create Zertifikatsrequest (CSR)
        subject = {'country_name': u'DE',
                   'state_name': u'Bundesland',
                   'locality_name': u'Stadt',
                   'organization_name': u'Testinstallation Eins CA',
                   'common_name': u'TestUser'}

        csr = x509.CertificateSigningRequestBuilder().subject_name(
            x509.Name([
                x509.NameAttribute(NameOID.COUNTRY_NAME, subject['country_name']),
                x509.NameAttribute(NameOID.STATE_OR_PROVINCE_NAME, subject['state_name']),
                x509.NameAttribute(NameOID.LOCALITY_NAME, subject['locality_name']),
                x509.NameAttribute(NameOID.ORGANIZATION_NAME, subject['organization_name']),
                x509.NameAttribute(NameOID.COMMON_NAME, subject['common_name']),
            ])).add_extension(
            x509.SubjectAlternativeName([]),
            critical=False
        ).sign(private_key, hashes.SHA256(), backend=default_backend())

        with open('DFN-PKI-Client_csr.pem', 'wb') as csr_file:
            csr_file.write(csr.public_bytes(serialization.Encoding.PEM))

        pkcs10 = csr.public_bytes(serialization.Encoding.PEM).decode()

        return pkcs10



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['dfn_pki_client']

package_data = \
{'': ['*']}

modules = \
['README', 'config.ini']
install_requires = \
['click>=7.1.2,<8.0.0',
 'cryptography>=3.4.5,<4.0.0',
 'pyOpenSSL>=20.0.1,<21.0.0',
 'suds-community>=0.8.4,<0.9.0',
 'urllib3>=1.26.3,<2.0.0']

entry_points = \
{'console_scripts': ['trackdownchanges = dfn_pki_client.main:main']}

setup_kwargs = {
    'name': 'dfn-pki-client',
    'version': '0.1.2',
    'description': 'Python module for the DFN-PKI soap API',
    'long_description': "# DFN-PKI-Client\n\n## Example\n\n```\n#!/usr/bin/env python\n\nfrom dfn_pki_client.public_service import PublicServicePKI\nfrom dfn_pki_client.registration_service import RegistrationService\nfrom dfn_pki_client.utils import get_wsdl\n\n\ndef main():\n\n    pki = PublicServicePKI('config.ini')\n\n    ca_info = pki.get_ca_info(42)\n\n    print(ca_info)\n\n\nif __name__ == '__main__':\n    main()\n\n```\n\n## Installation\n\n### Linux (e.g. Ubuntu)\n\n    apt install openssl rustc\n    pip3 install cryptography pyopenssl suds-community urllib3\n\n### macOS\n\n    brew install openssl swig rustup\n    pip3 install cryptography pyopenssl suds-community urllib3\n\n### Windows (untested)\n\n    pip3 install cryptography pyopenssl suds-community urllib3\n\n\n## Setup\n\nCreate the configuration file ``config.ini``\n\n```\n[default]\ncert = file_name.p12\npassword = 0123456789\npublic_wsdl = https://pki.pca.dfn.de/<ca_name>/cgi-bin/pub/soap?wsdl=1\nregistration_wsdl = https://ra.pca.dfn.de/<ca_name>/cgi-bin/ra/soap?wsdl=1\ndomain_wsdl = https://ra.pca.dfn.de/<ca_name>/cgi-bin/ra/soap/DFNCERT/Domains?wsdl=1'\n```\n",
    'author': 'Robert Grätz',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ikreb7/DFN-PKI-Client',
    'packages': packages,
    'package_data': package_data,
    'py_modules': modules,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
